package gr.uoc.csd.hy539.helloworld;

import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

import android.net.wifi.ScanResult;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;
import android.view.Menu;
import android.widget.TextView;

/*
 * Step 4 - perform 802.11 RSSI measurements.
 * 
 * This is a more advanced task.
 * 
 * We need user permission to access the WifiManager (a Service from the Application
 * Framework Layer) and get the ScanResults that it has available.
 * 
 * We also need user permission to force the WifiManager to get new ScanResults as soon
 * as possible.
 * 
 * The above two permissions are declared in the AndroidManifest.xml file.
 * 
 * Synoptically, this class should:
 * 		1) onCreate it should get a reference to the WifiManager object, in order to call
 * 		   its public methods.
 * 		2) periodically force the WifiManger to perform new scans (e.g., using a Timer)
 * 		3) register a BroadcastReceiver to listen for WifiManager broadcasts, in order to
 * 		   know when new scan results are available.
 *  
 */
public class WifiScanActivity extends Activity {
	final int refreshInterval= 100; // milliseconds between scans
	
	TextView textView1, textView2;
	private WifiManager wifiManager; // from Application Framework layer
	private ArrayList<ScanResult> scanResultList;
	private String string;
	private Timer timer; // it will periodically run the scanTask
	private long previousScanResultsTime; // for calculating the interval between scans 
	
	/*
	 * The task of forcing the WifiManger to perform new scans. The code here will be
	 * executed periodically, every time the timer fires.
	 */
	private TimerTask scanTask = new TimerTask() {
	    @Override
	    public void run() {
	        Log.i(this.getClass().getName(), "Timer task doing work");
	        wifiManager.startScan();
	    }
	};
	
	/* 
	 * This BroadcastReceiver will be registered for "SCAN_RESULTS_AVAILABLE_ACTION"
	 * broadcasts, broadcasted by the WifiManger.
	 * 
	 * Every time it receives a new broadcast, it will get the new scan results and
	 * display them on screen. It also calculates the delay between successive
	 * broadcasts.
	 * 
	 * Can you make this BroadcastReceiver write the new scan results in a file using
	 * CSV format with columns (Timestamp, MAC_address, RSSI) ?
	 */
	private BroadcastReceiver wifiReceiver = new BroadcastReceiver() {		
		@Override
		public void onReceive(Context context, Intent intent) {
			long scanResultsTime= System.currentTimeMillis();
			long dt= scanResultsTime -previousScanResultsTime;
			textView1.setText("Interval= "+dt+" ms");
			previousScanResultsTime= scanResultsTime;
			scanResultList = (ArrayList<ScanResult>) wifiManager.getScanResults();
	        string= "";
			for (ScanResult scanResult : scanResultList) {
				string+= scanResult.BSSID+" "+scanResult.level+"\n";
			}
			textView2.setText(string);
		}
	};

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_wifi_scan);
        textView1= (TextView) findViewById(R.id.wifiScanTextView1);
        textView2= (TextView) findViewById(R.id.wifiScanTextView2);
        
        previousScanResultsTime=System.currentTimeMillis();
		timer= new Timer();
		timer.schedule(scanTask, 0, refreshInterval);
		
		// get the WifiManager object
	    wifiManager= (WifiManager) getSystemService(Context.WIFI_SERVICE);
	    
	    // register the BroadcastReceiver for specific intents-events.
	    registerReceiver(wifiReceiver, new IntentFilter(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION));
    }
    
    /*
     * Before this Activity dies, it should unregister its BroadcastReceiver and
     * release the Timer's Thread.
     */
    @Override
	public void onDestroy() {
	    super.onDestroy();
	    unregisterReceiver(wifiReceiver);
	    if (timer != null){
	        timer.cancel();
	        timer.purge();
	        timer = null;
	    }
	}

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_wifi_scan, menu);
        return true;
    }
}
